@@ -6,9 +6,9 @@
 // Apache License 2.0
 //
 
-// This is needed on macOS to get the ncurses widechar API, and pkg-config fails to
-// define it.
-#ifdef __APPLE__
+// This is needed on FreeBSD and macOS to get the ncurses widechar API,
+// and pkg-config fails to define it.
+#if defined(__APPLE__) || defined(__FreeBSD__)
 #define _XOPEN_SOURCE_EXTENDED
 #else
 // This is needed for musl libc
@@ -41,13 +41,6 @@
 #include <err.h>
 #endif
 
-#define STR_(x) #x
-#define STR(x) STR_(x)
-#define VERSION_MAJOR 1
-#define VERSION_MINOR 7
-#define VERSION_PATCH 2
-#define VERSION_STR STR(VERSION_MAJOR) "." STR(VERSION_MINOR) "." STR(VERSION_PATCH)
-
 #define T_RARR '>'
 #define T_UARR '^'
 #ifdef NOACS
@@ -60,6 +53,27 @@
 #define T_LLCR ACS_LLCORNER
 #endif
 
+// Define standard curses color constants for better readability
+#define C_BLACK 0
+#define C_RED 1
+#define C_GREEN 2
+#define C_YELLOW 3
+#define C_BLUE 4
+#define C_MAGENTA 5
+#define C_CYAN 6
+#define C_WHITE 7
+
+// Define color element indices
+enum ColorElement {
+    LINE_COLOR = 0,
+    AXES_COLOR,
+    TEXT_COLOR,
+    TITLE_COLOR,
+    MAX_ERROR_COLOR,
+    MIN_ERROR_COLOR,
+    NUM_COLOR_ELEMENTS
+};
+
 enum Event {
     // These are made to have no set bits overlap to ease flag set testing
     EVENT_TIMEOUT = 1 << 0,
@@ -81,8 +95,8 @@ static int width = 0, height = 0, n = -1, v = 0, c = 0, rate = 0, two = 0,
 static bool fake_clock = false;
 static char *errstr = NULL;
 static bool redraw_needed = false;
-// Color indices: 0=plot_line, 1=axes, 2=text, 3=title
-static int colors[4] = {-1, -1, -1, -1};  // -1 means no color specified
+// Array of colors for different elements, -1 means no color specified
+static int colors[NUM_COLOR_ELEMENTS] = {-1, -1, -1, -1, -1, -1};
 static const char *verstring = "https://github.com/tenox7/ttyplot " VERSION_STR;
 
 static void usage(void) {
@@ -107,24 +121,69 @@ static void usage(void) {
         "lower-limit of the plot scale is fixed\n"
         "  -t title of the plot\n"
         "  -u unit displayed beside vertical bar\n"
-        "  -C color[,axes,text,title]  set colors (0-7) for elements:\n"
+        "  -C color[,axes,text,title,max_err,min_err]  set colors (0-7) for elements:\n"
         "     First value: plot line color\n"
         "     Second value: axes color (optional)\n"
         "     Third value: text color (optional)\n"
         "     Fourth value: title color (optional)\n"
-        "     Example: -C 1,2,3,4 or -C 1,2 or -C 1\n"
+        "     Fifth value: max error indicator color (optional)\n"
+        "     Sixth value: min error indicator color (optional)\n"
+        "     Example: -C 1,2,3,4,5,6 or -C 1,2 or -C 1\n"
+        "     Predefined color schemes:\n"
+        "       -C dark1    Blue-cyan-yellow scheme for dark terminals\n"
+        "       -C dark2    Purple-yellow-green scheme for dark terminals\n"
+        "       -C light1   Green-blue-red scheme for light terminals\n"
+        "       -C light2   Blue-green-yellow scheme for light terminals\n"
         "  -v print the current version and exit\n"
         "  -h print this help message and exit\n"
         "\n"
         "Hotkeys:\n"
         "   q quit\n"
-        "   r toggle rate mode\n");
+        "   r toggle rate mode\n"
+        "  ^L full screen refresh\n");
 }
 
 static void version(void) {
     printf("ttyplot %s\n", VERSION_STR);
 }
 
+// Set a predefined color scheme
+static void set_color_scheme(const char *scheme_name) {
+    if (strcmp(scheme_name, "dark1") == 0) {
+        // Blue-cyan-yellow scheme for dark terminals
+        colors[LINE_COLOR] = C_BLUE;        // Blue for plot line
+        colors[AXES_COLOR] = C_CYAN;        // Cyan for axes
+        colors[TEXT_COLOR] = C_WHITE;       // White for text
+        colors[TITLE_COLOR] = C_YELLOW;     // Yellow for title
+        colors[MAX_ERROR_COLOR] = C_RED;    // Red for max error
+        colors[MIN_ERROR_COLOR] = C_GREEN;  // Green for min error
+    } else if (strcmp(scheme_name, "dark2") == 0) {
+        // Purple-yellow-green scheme for dark terminals
+        colors[LINE_COLOR] = C_MAGENTA;    // Magenta for plot line
+        colors[AXES_COLOR] = C_YELLOW;     // Yellow for axes
+        colors[TEXT_COLOR] = C_CYAN;       // Cyan for text
+        colors[TITLE_COLOR] = C_GREEN;     // Green for title
+        colors[MAX_ERROR_COLOR] = C_RED;   // Red for max error
+        colors[MIN_ERROR_COLOR] = C_BLUE;  // Blue for min error
+    } else if (strcmp(scheme_name, "light1") == 0) {
+        // Green-blue-red scheme for light terminals
+        colors[LINE_COLOR] = C_GREEN;         // Green for plot line
+        colors[AXES_COLOR] = C_BLUE;          // Blue for axes
+        colors[TEXT_COLOR] = C_BLACK;         // Black for text
+        colors[TITLE_COLOR] = C_RED;          // Red for title
+        colors[MAX_ERROR_COLOR] = C_RED;      // Red for max error
+        colors[MIN_ERROR_COLOR] = C_MAGENTA;  // Magenta for min error
+    } else if (strcmp(scheme_name, "light2") == 0) {
+        // Blue-green-yellow scheme for light terminals
+        colors[LINE_COLOR] = C_BLUE;          // Blue for plot line
+        colors[AXES_COLOR] = C_GREEN;         // Green for axes
+        colors[TEXT_COLOR] = C_BLACK;         // Black for text
+        colors[TITLE_COLOR] = C_YELLOW;       // Yellow for title
+        colors[MAX_ERROR_COLOR] = C_RED;      // Red for max error
+        colors[MIN_ERROR_COLOR] = C_MAGENTA;  // Magenta for min error
+    }
+}
+
 // Replace *v1 and *v2 (if non-NULL) by their time derivatives.
 //  - v1, v2: addresses of input data and storage for results
 //  - now: current time
@@ -176,8 +235,8 @@ static void getminmax(int pw, double *values, double *min, double *max, double *
 
 static void draw_axes(int h, int ph, int pw, double max, double min, char *unit) {
     // Apply axes color if specified
-    if (colors[1] != -1)
-        attron(COLOR_PAIR(2));
+    if (colors[AXES_COLOR] != -1)
+        attron(COLOR_PAIR(AXES_COLOR + 1));
 
     // Draw axes
     mvhline(h - 3, 2, T_HLINE, pw);
@@ -186,56 +245,152 @@ static void draw_axes(int h, int ph, int pw, double max, double min, char *unit)
     mvaddch(1, 2, T_UARR);
     mvaddch(h - 3, 2, T_LLCR);
 
-    if (colors[1] != -1)
-        attroff(COLOR_PAIR(2));
+    if (colors[AXES_COLOR] != -1)
+        attroff(COLOR_PAIR(AXES_COLOR + 1));
 
     // Apply text color for scale labels if specified
-    if (colors[2] != -1)
-        attron(COLOR_PAIR(3));
+    if (colors[TEXT_COLOR] != -1)
+        attron(COLOR_PAIR(TEXT_COLOR + 1));
 
     // Print scale labels
     if (max - min >= 0.1) {
         mvprintw(1, 4, "%.1f %s", max, unit);
-        mvprintw((ph / 4) + 1, 4, "%.1f %s", min / 4 + max * 3 / 4, unit);
-        mvprintw((ph / 2) + 1, 4, "%.1f %s", min / 2 + max / 2, unit);
-        mvprintw((ph * 3 / 4) + 1, 4, "%.1f %s", min * 3 / 4 + max / 4, unit);
+
+        double label_val;
+
+        label_val = min / 4 + max * 3 / 4;
+        if (fabs(label_val) < 0.01)
+            label_val = 0.0;  // Prevent -0.0
+        mvprintw((ph / 4) + 1, 4, "%.1f %s", label_val, unit);
+
+        label_val = min / 2 + max / 2;
+        if (fabs(label_val) < 0.01)
+            label_val = 0.0;  // Prevent -0.0
+        mvprintw((ph / 2) + 1, 4, "%.1f %s", label_val, unit);
+
+        label_val = min * 3 / 4 + max / 4;
+        if (fabs(label_val) < 0.01)
+            label_val = 0.0;  // Prevent -0.0
+        mvprintw((ph * 3 / 4) + 1, 4, "%.1f %s", label_val, unit);
     }
 
-    if (colors[2] != -1)
-        attroff(COLOR_PAIR(3));
+    if (colors[TEXT_COLOR] != -1)
+        attroff(COLOR_PAIR(TEXT_COLOR + 1));
 }
 
 static void draw_line(int x, int ph, int l1, int l2, cchar_t *c1, cchar_t *c2,
-                      cchar_t *hce, cchar_t *lce) {
+                      cchar_t *hce, cchar_t *lce, int zero_pos, double v1, double v2,
+                      int has_v2) {
     static cchar_t space = {.attr = A_REVERSE, .chars = {' ', '\0'}};
     cchar_t c1r = *c1, c2r = *c2;
     c1r.attr |= A_REVERSE;
     c2r.attr |= A_REVERSE;
 
-    if (colors[0] != -1) {
-        c1->attr |= COLOR_PAIR(1);
-        c2->attr |= COLOR_PAIR(1);
-        c1r.attr |= COLOR_PAIR(1);
-        c2r.attr |= COLOR_PAIR(1);
-        space.attr |= COLOR_PAIR(1);
+    // Apply appropriate colors based on character type
+    if (c1 == hce && colors[MAX_ERROR_COLOR] != -1) {
+        // Max error indicator
+        c1->attr |= COLOR_PAIR(MAX_ERROR_COLOR + 1);
+        c1r.attr |= COLOR_PAIR(MAX_ERROR_COLOR + 1);
+    } else if (c1 == lce && colors[MIN_ERROR_COLOR] != -1) {
+        // Min error indicator
+        c1->attr |= COLOR_PAIR(MIN_ERROR_COLOR + 1);
+        c1r.attr |= COLOR_PAIR(MIN_ERROR_COLOR + 1);
+    } else if (colors[LINE_COLOR] != -1) {
+        // Normal plot line
+        c1->attr |= COLOR_PAIR(LINE_COLOR + 1);
+        c1r.attr |= COLOR_PAIR(LINE_COLOR + 1);
     }
 
-    if (l1 > l2) {
-        mvvline_set(ph + 1 - l1, x, c1, l1 - l2);
-        mvvline_set(ph + 1 - l2, x, &c2r, l2);
-    } else if (l1 < l2) {
-        mvvline_set(ph + 1 - l2, x, (c2 == hce || c2 == lce) ? &c2r : &space, l2 - l1);
-        mvvline_set(ph + 1 - l1, x, &c2r, l1);
+    if (c2 == hce && colors[MAX_ERROR_COLOR] != -1) {
+        // Max error indicator
+        c2->attr |= COLOR_PAIR(MAX_ERROR_COLOR + 1);
+        c2r.attr |= COLOR_PAIR(MAX_ERROR_COLOR + 1);
+    } else if (c2 == lce && colors[MIN_ERROR_COLOR] != -1) {
+        // Min error indicator
+        c2->attr |= COLOR_PAIR(MIN_ERROR_COLOR + 1);
+        c2r.attr |= COLOR_PAIR(MIN_ERROR_COLOR + 1);
+    } else if (colors[LINE_COLOR] != -1) {
+        // Normal plot line
+        c2->attr |= COLOR_PAIR(LINE_COLOR + 1);
+        c2r.attr |= COLOR_PAIR(LINE_COLOR + 1);
+    }
+
+    // Space always uses plot line color
+    if (colors[LINE_COLOR] != -1) {
+        space.attr |= COLOR_PAIR(LINE_COLOR + 1);
+    }
+
+    // Handle drawing based on whether values are positive or negative
+    if (zero_pos > 0) {  // We have negative values
+        int y1_start, y1_end, y2_start, y2_end;
+
+        // For value 1
+        if (v1 >= 0) {
+            // Positive value: draw from zero line upward
+            y1_start = ph + 1 - l1;
+            y1_end = ph + 1 - zero_pos;
+        } else {
+            // Negative value: draw from zero line downward
+            y1_start = ph + 1 - zero_pos;
+            y1_end = ph + 1 - l1;
+        }
+
+        // For value 2
+        if (has_v2) {
+            if (v2 > 0) {
+                y2_start = ph + 1 - l2;
+                y2_end = ph + 1 - zero_pos;
+            } else if (v2 < 0) {
+                y2_start = ph + 1 - zero_pos;
+                y2_end = ph + 1 - l2;
+            } else {  // v2 == 0
+                y2_start = ph + 1 - zero_pos;
+                y2_end = ph + 1 - zero_pos;
+            }
+        }
+
+        // Draw the lines
+        if (y1_start < y1_end) {
+            mvvline_set(y1_start, x, c1, y1_end - y1_start);
+        } else if (y1_start > y1_end && l1 > 0) {
+            mvvline_set(y1_end, x, c1, y1_start - y1_end);
+        }
+        if (has_v2) {
+            if (y2_start < y2_end) {
+                mvvline_set(y2_start, x, &c2r, y2_end - y2_start);
+            } else if (y2_start > y2_end) {
+                mvvline_set(y2_end, x, &c2r, y2_start - y2_end);
+            } else {  // y2_start == y2_end
+                mvvline_set(y2_start, x, &c2r, 1);
+            }
+        }
     } else {
-        mvvline_set(ph + 1 - l2, x, &c2r, l2);
+        // Original behavior for all positive values
+        if (l1 > l2) {
+            mvvline_set(ph + 1 - l1, x, c1, l1 - l2);
+            mvvline_set(ph + 1 - l2, x, &c2r, l2);
+        } else if (l1 < l2) {
+            mvvline_set(ph + 1 - l2, x, (c2 == hce || c2 == lce) ? &c2r : &space,
+                        l2 - l1);
+            mvvline_set(ph + 1 - l1, x, &c2r, l1);
+        } else {
+            mvvline_set(ph + 1 - l2, x, &c2r, l2);
+        }
     }
 
-    if (colors[0] != -1) {
-        c1->attr &= ~COLOR_PAIR(1);
-        c2->attr &= ~COLOR_PAIR(1);
-        c1r.attr &= ~COLOR_PAIR(1);
-        c2r.attr &= ~COLOR_PAIR(1);
-        space.attr &= ~COLOR_PAIR(1);
+    // Reset all color attributes (COLOR_PAIR indexes are LINE_COLOR+1 through
+    // MIN_ERROR_COLOR+1)
+    const attr_t color_mask = COLOR_PAIR(LINE_COLOR + 1) |
+                              COLOR_PAIR(MAX_ERROR_COLOR + 1) |
+                              COLOR_PAIR(MIN_ERROR_COLOR + 1);
+
+    c1->attr &= ~color_mask;
+    c2->attr &= ~color_mask;
+    c1r.attr &= ~color_mask;
+    c2r.attr &= ~color_mask;
+
+    if (colors[LINE_COLOR] != -1) {
+        space.attr &= ~COLOR_PAIR(LINE_COLOR + 1);
     }
 }
 
@@ -246,9 +401,17 @@ static void plot_values(int ph, int pw, double *v1, double *v2, double max, doub
     int i = (n + 1) % pw;
     int x;
     int l1, l2;
+    int zero_pos = 0;
 
-    if (colors[0] != -1)
-        attron(COLOR_PAIR(1));
+    // Calculate zero position if we have negative values
+    if (min < 0 && max > 0) {
+        zero_pos = lrint((0 - min) / (max - min) * ph);
+    } else if (max <= 0) {
+        zero_pos = ph;  // All values are negative, zero is at top
+    }
+
+    if (colors[LINE_COLOR] != -1)
+        attron(COLOR_PAIR(LINE_COLOR + 1));
 
     for (x = first_col; x < first_col + pw; x++, i = (i + 1) % pw) {
         /* suppress drawing uninitialized entries */
@@ -278,11 +441,12 @@ static void plot_values(int ph, int pw, double *v1, double *v2, double max, doub
                   (v2 && v2[i] > hardmax)   ? hce
                   : (v2 && v2[i] < hardmin) ? lce
                                             : pc,
-                  hce, lce);
+                  hce, lce, zero_pos, v1[i], (v2 && !isnan(v2[i])) ? v2[i] : 0,
+                  (v2 && !isnan(v2[i])));
     }
 
-    if (colors[0] != -1)
-        attroff(COLOR_PAIR(1));
+    if (colors[LINE_COLOR] != -1)
+        attroff(COLOR_PAIR(LINE_COLOR + 1));
 }
 
 static void show_all_centered(const char *message) {
@@ -291,13 +455,13 @@ static void show_all_centered(const char *message) {
     const int y = height / 2;
 
     // Apply title color to error messages if specified
-    if (colors[3] != -1)
-        attron(COLOR_PAIR(4));
+    if (colors[TITLE_COLOR] != -1)
+        attron(COLOR_PAIR(TITLE_COLOR + 1));
 
     mvaddnstr(y, x, message, width);
 
-    if (colors[3] != -1)
-        attroff(COLOR_PAIR(4));
+    if (colors[TITLE_COLOR] != -1)
+        attroff(COLOR_PAIR(TITLE_COLOR + 1));
 }
 
 static int window_big_enough_to_draw(void) {
@@ -337,8 +501,8 @@ static void paint_plot(void) {
         min = hardmin;
 
     // Apply text color if specified
-    if (colors[2] != -1)
-        attron(COLOR_PAIR(3));
+    if (colors[TEXT_COLOR] != -1)
+        attron(COLOR_PAIR(TEXT_COLOR + 1));
 
     mvaddstr(height - 1, width - strlen(verstring) - 1, verstring);
 
@@ -352,12 +516,12 @@ static void paint_plot(void) {
     }
     mvaddstr(height - 2, width - strlen(clock_display), clock_display);
 
-    if (colors[2] != -1)
-        attroff(COLOR_PAIR(3));
+    if (colors[TEXT_COLOR] != -1)
+        attroff(COLOR_PAIR(TEXT_COLOR + 1));
 
     // Apply text color for stats
-    if (colors[2] != -1)
-        attron(COLOR_PAIR(3));
+    if (colors[TEXT_COLOR] != -1)
+        attron(COLOR_PAIR(TEXT_COLOR + 1));
 
     mvvline_set(height - 2, 5, &plotchar, 1);
     if (v > 0) {
@@ -374,8 +538,8 @@ static void paint_plot(void) {
         }
     }
 
-    if (colors[2] != -1)
-        attroff(COLOR_PAIR(3));
+    if (colors[TEXT_COLOR] != -1)
+        attroff(COLOR_PAIR(TEXT_COLOR + 1));
 
     plot_values(plotheight, plotwidth, values1, two ? values2 : NULL, max, min, n,
                 &plotchar, &max_errchar, &min_errchar, hardmax, hardmin);
@@ -383,13 +547,13 @@ static void paint_plot(void) {
     draw_axes(height, plotheight, plotwidth, max, min, unit);
 
     // Apply title color if specified
-    if (colors[3] != -1)
-        attron(COLOR_PAIR(4));
+    if (colors[TITLE_COLOR] != -1)
+        attron(COLOR_PAIR(TITLE_COLOR + 1));
 
     mvaddstr(0, (width / 2) - (strlen(title) / 2), title);
 
-    if (colors[3] != -1)
-        attroff(COLOR_PAIR(4));
+    if (colors[TITLE_COLOR] != -1)
+        attroff(COLOR_PAIR(TITLE_COLOR + 1));
 
     move(0, 0);
 }
@@ -401,9 +565,11 @@ static void signal_handler(int signum) {
     const unsigned char signal_number =
         (unsigned char)signum;  // signum is either 2 (SIGINT) or 28 (SIGWINCH)
     ssize_t write_res;
+    int saved_errno = errno;
     do {
         write_res = write(signal_write_fd, &signal_number, 1);
     } while ((write_res == -1) && (errno == EINTR));
+    errno = saved_errno;
 }
 
 static void redraw_screen(const char *errstr) {
@@ -705,16 +871,23 @@ int main(int argc, char *argv[]) {
                 mbtowc(&min_errchar.chars[0], optarg, MB_CUR_MAX);
                 break;
             case 'C': {
-                char *color_str = strdup(optarg);
-                char *token = strtok(color_str, ",");
-                int color_idx = 0;
+                // Check if it's a predefined color scheme
+                if (strcmp(optarg, "dark1") == 0 || strcmp(optarg, "dark2") == 0 ||
+                    strcmp(optarg, "light1") == 0 || strcmp(optarg, "light2") == 0) {
+                    set_color_scheme(optarg);
+                } else {
+                    // Process comma-separated color values
+                    char *color_str = strdup(optarg);
+                    char *token = strtok(color_str, ",");
+                    int color_idx = 0;
 
-                while (token != NULL && color_idx < 4) {
-                    colors[color_idx++] = atoi(token);
-                    token = strtok(NULL, ",");
+                    while (token != NULL && color_idx < NUM_COLOR_ELEMENTS) {
+                        colors[color_idx++] = atoi(token);
+                        token = strtok(NULL, ",");
+                    }
+
+                    free(color_str);
                 }
-
-                free(color_str);
                 break;
             }
             case 's':
@@ -745,7 +918,10 @@ int main(int argc, char *argv[]) {
     if (hardmax <= hardmin)
         hardmax = FLT_MAX;
 
-    initscr(); /* uses filesystem, so before pledge */
+    if (initscr() == NULL) {
+        fprintf(stderr, "Error: failed to initialize ncurses\n");
+        exit(1);
+    }
 
 #ifdef __OpenBSD__
     if (pledge("stdio tty", NULL) == -1)
@@ -754,7 +930,7 @@ int main(int argc, char *argv[]) {
 
     // Check if any colors are defined
     bool has_colors = false;
-    for (int i = 0; i < 4; i++) {
+    for (int i = 0; i < NUM_COLOR_ELEMENTS; i++) {
         if (colors[i] != -1) {
             has_colors = true;
             break;
@@ -766,12 +942,15 @@ int main(int argc, char *argv[]) {
         use_default_colors();
 
         // Initialize color pairs for different elements
-        // COLOR_PAIR(1): plot line (initialized in original code)
-        // COLOR_PAIR(2): axes
-        // COLOR_PAIR(3): text
-        // COLOR_PAIR(4): title
+        // COLOR_PAIR indexes match the enum + 1 because ncurses starts at 1
+        // COLOR_PAIR(1): plot line (LINE_COLOR + 1)
+        // COLOR_PAIR(2): axes (AXES_COLOR + 1)
+        // COLOR_PAIR(3): text (TEXT_COLOR + 1)
+        // COLOR_PAIR(4): title (TITLE_COLOR + 1)
+        // COLOR_PAIR(5): max error indicator (MAX_ERROR_COLOR + 1)
+        // COLOR_PAIR(6): min error indicator (MIN_ERROR_COLOR + 1)
 
-        for (int i = 0; i < 4; i++) {
+        for (int i = 0; i < NUM_COLOR_ELEMENTS; i++) {
             if (colors[i] != -1) {
                 init_pair(i + 1, colors[i], -1);  // -1 for default background
             }
@@ -847,6 +1026,10 @@ int main(int argc, char *argv[]) {
                     rate = ! rate;
                 else if (key == 'q')  // 'q' = quit
                     break;
+                else if (key == '\f' || key == 12) {  // Ctrl+L = full screen refresh
+                    clear();
+                    redraw_needed = true;
+                }
             } else if (count == 0) {
                 close(tty);
                 tty = -1;
